@@ -1,8 +1,7 @@
-use crate::{Queryable, TreeIter, Value, ValueConvertError, ValueIter};
+use crate::{Queryable, TreeIter, Value, ValueConvertError};
 use std::borrow::Cow;
 use std::collections::*;
 use std::convert::{TryFrom, TryInto};
-use std::fmt;
 use std::hash::Hash;
 
 macro_rules! value_int {
@@ -44,6 +43,7 @@ macro_rules! value_int {
                         } else {
                             0.try_into().map_err(|_| ValueConvertError)
                         }
+                        Value::Array(_) => Err(ValueConvertError),
                     }
                 }
             }
@@ -114,12 +114,20 @@ impl From<&String> for Value {
     }
 }
 
-impl From<Value> for String {
-    fn from(v: Value) -> String {
+impl TryFrom<Value> for String {
+    type Error = ValueConvertError;
+    fn try_from(v: Value) -> Result<String, ValueConvertError> {
         match v {
-            Value::Int(i) => i.to_string(),
-            Value::String(s) => s,
-            Value::Bool(b) => b.to_string(),
+            Value::Int(i) => Ok(i.to_string()),
+            Value::String(s) => Ok(s),
+            Value::Bool(b) => Ok(b.to_string()),
+            Value::Array(arr) => {
+                if arr.0.len() == 1 {
+                    Ok(arr.0[0].to_string())
+                } else {
+                    Err(ValueConvertError)
+                }
+            }
         }
     }
 }
@@ -135,23 +143,6 @@ where
     }
 }
 
-impl fmt::Display for Value {
-    fn fmt(&self, f: &mut fmt::Formatter) -> fmt::Result {
-        match self {
-            Value::String(v) => write!(f, "{}", v),
-            Value::Int(v) => write!(f, "{}", v),
-            Value::Bool(v) => write!(f, "{}", v),
-        }
-    }
-}
-
-impl<'a> fmt::Debug for &'a dyn Queryable<'a> {
-    fn fmt(&self, f: &mut fmt::Formatter) -> fmt::Result {
-        let keys: Vec<_> = self.keys().collect();
-        write!(f, "{:?}", keys)
-    }
-}
-
 impl<'a> Queryable<'a> for bool {
     fn data(&self) -> Option<Value> {
         Some(self.into())
@@ -184,7 +175,7 @@ impl<'a> Queryable<'a> for String {
 
 impl<'a> Queryable<'a> for &String {
     fn data(&self) -> Option<Value> {
-        Some(self.clone().into())
+        Some((*self).into())
     }
 }
 
@@ -193,20 +184,15 @@ where
     // TODO get rid of the Clone constraint
     K: Queryable<'a> + Hash + Eq + Clone,
     V: Queryable<'a>,
-    Value: From<K>,
     K: TryFrom<Value>,
 {
-    fn keys(&'a self) -> ValueIter<'a> {
-        ValueIter(Box::from(self.keys().map(|k| k.clone().into())))
-    }
-
     fn member<'f>(&'a self, field: &'f Value) -> Option<&'a dyn Queryable<'a>> {
         let key = K::try_from(field.clone()).ok()?;
         self.get(&key).map(|v| v as _)
     }
 
     fn all(&'a self) -> TreeIter<'a> {
-        TreeIter(Box::new(self.values().map(|v| v as _)))
+        TreeIter::from_queryables(self.values())
     }
 }
 
@@ -214,20 +200,15 @@ impl<'a, K, V> Queryable<'a> for BTreeMap<K, V>
 where
     K: Queryable<'a> + Ord + Eq + 'a,
     V: Queryable<'a>,
-    Value: From<&'a K>,
     K: TryFrom<Value>,
 {
-    fn keys(&'a self) -> ValueIter<'a> {
-        ValueIter(Box::from(self.keys().map(|k| k.into())))
-    }
-
     fn member<'f>(&'a self, field: &'f Value) -> Option<&dyn Queryable<'a>> {
         let key = K::try_from(field.clone()).ok()?;
         self.get(&key).map(|v| v as _)
     }
 
     fn all(&'a self) -> TreeIter<'a> {
-        TreeIter(Box::new(self.values().map(|v| v as _)))
+        TreeIter::from_queryables(self.values())
     }
 }
 
@@ -236,7 +217,7 @@ where
     V: Queryable<'a> + Ord + Eq,
 {
     fn all(&'a self) -> TreeIter<'a> {
-        TreeIter(Box::new(self.iter().map(|v| v as _)))
+        TreeIter::from_queryables(self)
     }
 }
 
@@ -245,7 +226,7 @@ where
     V: Queryable<'a> + Hash + Eq,
 {
     fn all(&'a self) -> TreeIter<'a> {
-        TreeIter(Box::new(self.iter().map(|v| v as _)))
+        TreeIter::from_queryables(self)
     }
 }
 
@@ -253,17 +234,13 @@ impl<'a, T> Queryable<'a> for Vec<T>
 where
     T: Queryable<'a>,
 {
-    fn keys(&'a self) -> ValueIter<'a> {
-        ValueIter(Box::from((0..self.len()).map(|v| v.into())))
-    }
-
     fn member<'f>(&'a self, field: &'f Value) -> Option<&dyn Queryable<'a>> {
         let index = usize::try_from(field).ok()?;
         self.get(index).map(|v| v as _)
     }
 
     fn all(&'a self) -> TreeIter<'a> {
-        TreeIter(Box::new(self.iter().map(|v| v as _)))
+        TreeIter::from_queryables(self)
     }
 }
 
@@ -271,17 +248,13 @@ impl<'a, T> Queryable<'a> for VecDeque<T>
 where
     T: Queryable<'a>,
 {
-    fn keys(&'a self) -> ValueIter<'a> {
-        ValueIter(Box::from((0..self.len()).map(|v| v.into())))
-    }
-
     fn member<'f>(&'a self, field: &'f Value) -> Option<&dyn Queryable<'a>> {
         let index = usize::try_from(field).ok()?;
         self.get(index).map(|v| v as _)
     }
 
     fn all(&'a self) -> TreeIter<'a> {
-        TreeIter(Box::new(self.iter().map(|v| v as _)))
+        TreeIter::from_queryables(self.iter())
     }
 }
 
@@ -289,10 +262,6 @@ impl<'a, T> Queryable<'a> for Box<T>
 where
     T: Queryable<'a>,
 {
-    fn keys(&'a self) -> ValueIter<'a> {
-        self.as_ref().keys()
-    }
-
     fn member<'f>(&'a self, field: &'f Value) -> Option<&dyn Queryable<'a>> {
         self.as_ref().member(field)
     }
@@ -310,14 +279,6 @@ impl<'a, T> Queryable<'a> for Option<T>
 where
     T: Queryable<'a>,
 {
-    fn keys(&'a self) -> ValueIter<'a> {
-        if let Some(val) = self.as_ref() {
-            val.keys()
-        } else {
-            ValueIter::empty()
-        }
-    }
-
     fn member<'f>(&'a self, field: &'f Value) -> Option<&dyn Queryable<'a>> {
         self.as_ref().and_then(|val| val.member(field))
     }
@@ -326,7 +287,7 @@ where
         if let Some(val) = self.as_ref() {
             val.all()
         } else {
-            TreeIter(Box::from(std::iter::empty()))
+            TreeIter::empty()
         }
     }
 
@@ -339,14 +300,6 @@ impl<'a, T, E> Queryable<'a> for Result<T, E>
 where
     T: Queryable<'a>,
 {
-    fn keys(&'a self) -> ValueIter<'a> {
-        if let Ok(val) = self.as_ref() {
-            val.keys()
-        } else {
-            ValueIter::empty()
-        }
-    }
-
     fn member<'f>(&'a self, field: &'f Value) -> Option<&dyn Queryable<'a>> {
         self.as_ref().ok().and_then(|val| val.member(field))
     }
@@ -355,7 +308,7 @@ where
         if let Ok(val) = self.as_ref() {
             val.all()
         } else {
-            TreeIter(Box::from(std::iter::empty()))
+            TreeIter::empty()
         }
     }
 
@@ -374,10 +327,6 @@ impl<'a, T> Queryable<'a> for std::cmp::Reverse<T>
 where
     T: Queryable<'a>,
 {
-    fn keys(&'a self) -> ValueIter<'a> {
-        self.0.keys()
-    }
-
     fn member<'f>(&'a self, field: &'f Value) -> Option<&dyn Queryable<'a>> {
         self.0.member(field)
     }
@@ -394,18 +343,14 @@ where
 macro_rules! impl_tuples {
     ( => ) => {};
     ($t: ident $(,$other_t: ident)* => $ind: tt $(,$other_ind: tt)*) => {
-        impl_tuples!(@ $t $(, $other_t)* => $ind $(, $other_ind)*);
+        impl_tuples!(@ $t $(, $other_t)* ($ind) => $ind $(, $other_ind)*);
         impl_tuples!($($other_t),* => $($other_ind),*);
     };
-    (@ $($t: ident),+ => $($ind: tt),+) => {
+    (@ $($t: ident),+ ($len: tt) => $($ind: tt),+) => {
         impl<'a $(,$t)+> Queryable<'a> for ($($t,)+)
         where
             $($t: Queryable<'a>),+
         {
-            fn keys(&'a self) -> ValueIter<'a> {
-                ValueIter(Box::from(vec![$($ind),+].into_iter().map(Value::from)))
-            }
-
             fn member<'f>(&'a self, field: &'f Value) -> Option<&dyn Queryable<'a>> {
                 #[allow(clippy::collapsible_match)]
                 match field {
@@ -429,7 +374,29 @@ macro_rules! impl_tuples {
                 None
             }
         }
-    }
+
+        impl<$($t),+> TryFrom<Value> for ($($t,)+)
+        where
+            $($t: TryFrom<Value>),+
+        {
+            type Error = ValueConvertError;
+            fn try_from(value: Value) -> Result<Self, ValueConvertError> {
+                match value {
+                    #[allow(unused_comparisons)]
+                    Value::Array(arr) if arr.len() >= $len => {
+                        let mut iter = arr.into_iter();
+                        Ok((
+                            $(
+                                $t::try_from(iter.next().ok_or(ValueConvertError)?)
+                                    .map_err(|_| ValueConvertError)?,
+                            )+
+                        ))
+                    }
+                    _ => Err(ValueConvertError),
+                }
+            }
+        }
+    };
 }
 
 impl_tuples!(A, B, C, D, E, F, G, H, I, J, K, L => 11, 10, 9, 8, 7, 6, 5, 4, 3, 2, 1, 0);
@@ -445,12 +412,8 @@ macro_rules! impl_arrays {
         where
             T: Queryable<'a>
         {
-            fn keys(&'a self) -> ValueIter<'a> {
-                let vec: Vec<i64> = vec![$($ind),*];
-                ValueIter(Box::from(vec.into_iter().map(Value::from)))
-            }
-
             fn member<'f>(&'a self, field: &'f Value) -> Option<&dyn Queryable<'a>> {
+                #[allow(clippy::collapsible_match)]
                 match field {
                     Value::Int(i) => {
                         match i {
@@ -472,7 +435,35 @@ macro_rules! impl_arrays {
                 None
             }
         }
-    }
+
+        impl<T> TryFrom<Value> for [T; $size]
+        where
+            T: TryFrom<Value>,
+        {
+            type Error = ValueConvertError;
+            fn try_from(value: Value) -> Result<Self, ValueConvertError> {
+                match value {
+                    #[allow(unused_comparisons)]
+                    Value::Array(arr) if arr.len() >= $size => {
+                        #[allow(unused_variables, unused_mut)]
+                        let mut iter = arr.into_iter();
+                        Ok([
+                            $(
+                                impl_arrays!(@foreach ($ind) =>
+                                    T::try_from(iter.next().ok_or(ValueConvertError)?)
+                                        .map_err(|_| ValueConvertError)?
+                                )
+                            ),*
+                        ])
+                    }
+                    _ => Err(ValueConvertError),
+                }
+            }
+        }
+    };
+    (@foreach ($($counter: tt)*) => $($output: tt)*) => {
+        $($output)*
+    };
 }
 
 impl_arrays!(
